/**
 * Modular Multiplicative Inverse
 *
 * Complexity (Time): O(log m)
 * Complexity (Space): O(1)
 */

// ========== Fermat's Little Theorem ==========
// Used if m is prime

ll power(ll x, ll y) {
  ll ans = 1;

  while (y) {
    if (y & 1)
      ans = (ans * x) % MOD;

    y >>= 1;
    x = (x * x) % MOD;
  }

  return ans % MOD;
}


// Find x where:
// a*x === 1 (mod MOD)
ll mod_inverse(ll a) {
  return power(a, MOD - 2);
}


// ========== Extended Euclidean Algorithm ==========
// Used if m and a are coprime

// return gcd(a, b) and find x, y where:
// a*x + b*y = gcd(a, b)
ll gcd_extended(ll a, ll b, ll &x, ll &y) {
  if (!a) {
    x = 0;
    y = 1;
    return b;
  }

  ll x1, y1;
  ll g = gcd_extended(b % a, a, x1, y1);

  x = y1 - (b / a) * x1;
  y = x1;

  return g;
}


// Find x where:
// a*x === 1 (mod MOD)
ll mod_inverse(ll a) {
  ll x, y;
  ll g = gcd_extended(a, MOD, x, y);

  return (x % MOD + MOD) % MOD;
}
